package com.informagen.echo2;

import java.util.EventListener;

import nextapp.echo2.app.Alignment;
import nextapp.echo2.app.Color;
import nextapp.echo2.app.TextField;
import nextapp.echo2.app.text.Document;

import nextapp.echo2.app.event.ActionEvent;
import nextapp.echo2.app.event.ActionListener;


/**
 * A single-line text input field with pre keystroke regular expression testing
 */

abstract public class ActiveTextField extends TextField {


    public static final String INPUT_VALID = "inputValid";
    public static final String INPUT_INVALID = "inputInvalid";

    public static final String PROPERTY_MINVALUE = "minimumValue";
    public static final String PROPERTY_MAXVALUE = "maximumValue";
    public static final String PROPERTY_STATUSMSG = "message";
    
    public static final String PROPERTY_MSG_VISIBLE = "messageVisible";
    public static final String PROPERTY_MSG_ALWAYS_VISIBLE = "messageAlwaysVisible";
    public static final String PROPERTY_MSG_POSITION = "messagePosition";
    
    public static final String PROPERTY_ICON_VISIBLE = "iconVisible";
    public static final String PROPERTY_ICON_ALWAYS_VISIBLE = "iconAlwaysVisible";
    public static final String PROPERTY_ICON_POSITION = "iconPosition";

    public static final String PROPERTY_ERROR_FOREGROUND = "errorForeground";
    public static final String PROPERTY_ERROR_BACKGROUND = "errorBackground";


    /**
     * Creates a new <code>IntegerTextField</code> with an empty 
     * <code>StringDocument</code> as its model, and default width
     * setting.
    */
    public ActiveTextField() {}

    
    /**
     * Creates a new <code>IntegerTextField</code> with the specified
     * <code>Document</code> model.
     * 
     * @param document the document
    */

    public ActiveTextField(Document document) {
        super(document);
    }

    
    /**
     * Creates a new <code>IntegerTextField</code> with the specified
     * <code>Document</code> model, initial text, and column width.
     * 
     * @param document the document
     * @param text the initial text (may be null)
     * @param columns the number of columns to display
    */
     
    public ActiveTextField(Document document, String text, int columns) {
        super(document, text, columns);
    }

    /**
     * @see nextapp.echo2.app.Component#processInput(java.lang.String, java.lang.Object)
     */

    public void processInput(String inputName, Object inputValue) {
        super.processInput(inputName, inputValue);


        if (!hasEventListenerList()) {
            return;
        }
        
        EventListener[] listeners = getEventListenerList().getListeners(ActionListener.class);
 
        ActionEvent e = null;
        
        boolean valid = Boolean.valueOf((String)inputValue).booleanValue();
        
        
        for (int i = 0; i < listeners.length; ++i) {
            // Lazy instanciation
            if (e == null) 
                e = new ActionEvent(this, (valid) ? INPUT_VALID : INPUT_INVALID);
            
            ((ActionListener) listeners[i]).actionPerformed(e);
        }
    }



      
    /**
     * Set the status message 
     * 
     * @param minValue the minimum value
    */
       
    public void setMessage(String statusMsg) {
        setProperty(PROPERTY_STATUSMSG, statusMsg);
    }


    /**
     * Get the status, out of range, message
     * 
     * @returns String the status message, ie Valid values are...
    */
       
    public String getMessage() {
        return (String)getProperty(PROPERTY_STATUSMSG);
    }


    /**
     * Set the visibility of the status message
     * 
     * @param isVisible the visibility
    */
       
    public void setMessageVisible(boolean isVisible) {
        setProperty(PROPERTY_MSG_VISIBLE, new Boolean(isVisible));
    }


    /**
     * Get the visibility of the status message
     * 
     * @returns boolean the status message visibility
    */
       
    public boolean isMessageVisible() {
        Boolean object = (Boolean)getProperty(PROPERTY_MSG_VISIBLE);
        return (object != null) ? object.booleanValue() : true;
    }



    /**
     * Set the status message to be always visible
     * 
     * @param minValue the minimum value
    */
       
    public void setMessageAlwaysVisible(boolean isAlwaysVisible) {
        setProperty(PROPERTY_MSG_ALWAYS_VISIBLE, new Boolean(isAlwaysVisible));
    }


    /**
     * Should the status message always be visible
     * 
     * @returns String the status message, ie Valid values are...
    */
       
    public boolean isMessageAlwaysVisible() {
        Boolean object = (Boolean)getProperty(PROPERTY_MSG_ALWAYS_VISIBLE);
        return (object != null) ? object.booleanValue() : false;
    }


    /**
     * Set the position of the message 
     * 
     * @param messagePosition the status message position
    */
       
    public void setMessagePosition(Alignment messagePosition) {
        setProperty(PROPERTY_MSG_POSITION, messagePosition);
    }


    /**
     * Set the visibility of the status message
     * 
     * @param isVisible the visibility
    */
       
    public void setIconVisible(boolean isVisible) {
        setProperty(PROPERTY_ICON_VISIBLE, new Boolean(isVisible));
    }


    /**
     * Get the visibility of the status message
     * 
     * @returns boolean the status message visibility
    */
       
    public boolean isIconVisible() {
        Boolean object = (Boolean)getProperty(PROPERTY_ICON_VISIBLE);
        return (object != null) ? object.booleanValue() : true;
    }



    /**
     * Set the icon to be always visible
     * 
     * @param minValue the minimum value
    */
       
    public void setIconAlwaysVisible(boolean isAlwaysVisible) {
        setProperty(PROPERTY_ICON_ALWAYS_VISIBLE, new Boolean(isAlwaysVisible));
    }


    /**
     * Should the icon always be visible
     * 
     * @returns String the status message, ie Valid values are...
    */
       
    public boolean isIconAlwaysVisible() {
        Boolean object = (Boolean)getProperty(PROPERTY_ICON_ALWAYS_VISIBLE);
        return (object != null) ? object.booleanValue() : false;
    }



    /**
     * Get the icon position
     * 
     * @returns Alignment the icon position relative to the input field
    */
       
    public Alignment getMessagePosition() {
        return (Alignment)getProperty(PROPERTY_MSG_POSITION);
    }



    /**
     * Set the position of the icon 
     * 
     * @param iconPosition the status icon position
    */
       
    public void setIconPosition(Alignment messagePosition) {
        setProperty(PROPERTY_ICON_POSITION, messagePosition);
    }


    /**
     * Get the icon position
     * 
     * @returns Alignment the icon position relative to the input field
    */
       
    public Alignment getIconPosition() {
        return (Alignment)getProperty(PROPERTY_ICON_POSITION);
    }


    /**
     * Set the foreground color when input is out of range
     * 
     * @param foreground the error foreground color
    */
       
    public void setErrorForeground(Color color) {
        setProperty(PROPERTY_ERROR_FOREGROUND, color);
    }


    /**
     * Get the foreground color when input is out of range
     * 
     * @returns Color the error foreground color
    */
       
    public Color getErrorForeground() {
        return (Color)getProperty(PROPERTY_ERROR_FOREGROUND);
    }



    /**
     * Set the background color when input is out of range
     * 
     * @param background the error background color
    */
       
    public void setErrorBackground(Color color) {
        setProperty(PROPERTY_ERROR_BACKGROUND, color);
    }


    /**
     * Get the background color when input is out of range
     * 
     * @returns Color the error background color
    */
       
    public Color getErrorBackground() {
        return (Color)getProperty(PROPERTY_ERROR_BACKGROUND);
    }


}

