package com.informagen.echo2;

import com.informagen.echo2.ActiveTextField;

import java.lang.NumberFormatException;

/**
 * A single-line text input field with pre keystroke regular expression testing
 */

public class IntegerTextField extends ActiveTextField {

    public static final String PROPERTY_MINVALUE = "minimumValue";
    public static final String PROPERTY_MAXVALUE = "maximumValue";



    /**
     * Set the integer value as an int
     * 
     * @param value the int value
    */
       
    public void setValue(int value) {
        setText(Integer.toString(value));
    }


    /**
     * Get the value as an int
     * 
     * @returns int the int value
    */
       
    public int getValue() {
        int value;
        try {
            value = Integer.parseInt(getText());
        } catch (NumberFormatException nfe) {
            value = 0;
        }
        
        return value;
    }

    
    /**
     * Set the maximum allowed value
     * 
     * @param maxValue the maximum value
    */
       
    public void setMaxValue(int maxValue) {
        setProperty(PROPERTY_MAXVALUE, new Integer(maxValue));
    }


    /**
     * Get the maximum value for this integer field
     * 
     * @returns int the maximum value
    */
       
    public int getMaxValue() {
        Integer object = (Integer)getProperty(PROPERTY_MAXVALUE);
        return (object != null) ? object.intValue() : Integer.MAX_VALUE;
    }

    
    /**
     * Set the minimum allowed value 
     * 
     * @param minValue the minimum value
    */
       
    public void setMinValue(int minValue) {
        setProperty(PROPERTY_MINVALUE, new Integer(minValue));
    }


    /**
     * Get the minimum value for this integer field
     * 
     * @returns int the minimum value
    */
       
    public int getMinValue() {
        Integer object = (Integer)getProperty(PROPERTY_MINVALUE);
        return (object != null) ? object.intValue() : Integer.MIN_VALUE;
    }

}

