package com.informagen.echo2.peers;


// informagen Echo2 components
import com.informagen.echo2.ActiveTextArea;
import com.informagen.echo2.peers.partialupdate.TextUpdate;
import com.informagen.echo2.peers.partialupdate.ForegroundUpdate;
import com.informagen.echo2.peers.partialupdate.BackgroundUpdate;


// NextApp Echo2 Component Framework
import nextapp.echo2.app.Alignment;
import nextapp.echo2.app.FillImage;
import nextapp.echo2.app.Border;
import nextapp.echo2.app.Color;
import nextapp.echo2.app.Component;
import nextapp.echo2.app.Extent;
import nextapp.echo2.app.Font;
import nextapp.echo2.app.ImageReference;
import nextapp.echo2.app.Insets;
import nextapp.echo2.app.TextArea;
import nextapp.echo2.app.TextField;

import nextapp.echo2.app.text.TextComponent;

import nextapp.echo2.app.update.ServerComponentUpdate;

// NextApp Echo2 WebContainer Sub-Framework
import nextapp.echo2.webcontainer.ActionProcessor;
import nextapp.echo2.webcontainer.ContainerInstance;
import nextapp.echo2.webcontainer.DomUpdateSupport;
import nextapp.echo2.webcontainer.FocusSupport;
import nextapp.echo2.webcontainer.PartialUpdateManager;
import nextapp.echo2.webcontainer.PartialUpdateParticipant;
import nextapp.echo2.webcontainer.PropertyUpdateProcessor;
import nextapp.echo2.webcontainer.RenderContext;
import nextapp.echo2.webcontainer.ComponentSynchronizePeer;

import nextapp.echo2.webcontainer.image.ImageRenderSupport;

import nextapp.echo2.webcontainer.partialupdate.BorderUpdate;
import nextapp.echo2.webcontainer.partialupdate.ColorUpdate;
import nextapp.echo2.webcontainer.partialupdate.InsetsUpdate;

// NextApp Echo2 WebRenderer Sub-Framework

import nextapp.echo2.webrender.ClientProperties;
import nextapp.echo2.webrender.ServerMessage;
import nextapp.echo2.webrender.Service;
import nextapp.echo2.webrender.ServiceRegistry;
import nextapp.echo2.webrender.WebRenderServlet;

import nextapp.echo2.webrender.output.CssStyle;

import nextapp.echo2.webrender.servermessage.DomUpdate;
import nextapp.echo2.webrender.servermessage.WindowUpdate;

import nextapp.echo2.webrender.service.StaticBinaryService;

import nextapp.echo2.webrender.service.JavaScriptService;

// NextApp Echo2 WebRenderer CSS renderers
import nextapp.echo2.webcontainer.propertyrender.AlignmentRender;
import nextapp.echo2.webcontainer.propertyrender.FillImageRender;
import nextapp.echo2.webcontainer.propertyrender.BorderRender;
import nextapp.echo2.webcontainer.propertyrender.ColorRender;
import nextapp.echo2.webcontainer.propertyrender.ExtentRender;
import nextapp.echo2.webcontainer.propertyrender.FontRender;
import nextapp.echo2.webcontainer.propertyrender.InsetsRender;

import nextapp.echo2.webrender.util.DomUtil;


// W3C DOM
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;


/**
 * Abstract base synchronization peer for the built-in
 * <code>nextapp.echo2.app.text.TextComponent</code> -derived components.
 * <p>
 * This class should not be extended or used by classes outside of the Echo
 * framework.
 */
public class ActiveTextAreaPeer implements ActionProcessor, 
                                           ComponentSynchronizePeer, 
                                           DomUpdateSupport, 
                                           FocusSupport, 
                                           ImageRenderSupport, 
                                           PropertyUpdateProcessor {


    private static final String IMAGE_ID_BACKGROUND = "background";

    /**
     * Service to provide supporting JavaScript library.
     */
    static final Service ACTIVE_TEXTAREA_SERVICE = JavaScriptService.forResource(
                    "Echo.ActiveTextArea",
                    "/com/informagen/echo2/resources/js/ActiveTextArea.js");


    static {
		ServiceRegistry registery = WebRenderServlet.getServiceRegistry();
        registery.add(ACTIVE_TEXTAREA_SERVICE);
    }
 
 
    private PartialUpdateManager partialUpdateManager;
    
    private final String messageProcessor = "ActiveTextArea.MessageProcessor";

    /**
     * Default constructor.
     */

    public ActiveTextAreaPeer() {
        
        partialUpdateManager = new PartialUpdateManager();
        
        //partialUpdateManager.add(Component.PROPERTY_FOREGROUND,
        //        new ColorUpdate(Component.PROPERTY_FOREGROUND, ".textarea", ColorUpdate.CSS_COLOR));
                
        //partialUpdateManager.add(Component.PROPERTY_BACKGROUND,
        //        new ColorUpdate(Component.PROPERTY_BACKGROUND, ".textarea", ColorUpdate.CSS_BACKGROUND_COLOR));
                
        partialUpdateManager.add(TextField.PROPERTY_BORDER,
                new BorderUpdate(TextField.PROPERTY_BORDER, ".textarea", BorderUpdate.CSS_BORDER));
        
        partialUpdateManager.add(Component.PROPERTY_FOREGROUND, new ForegroundUpdate(messageProcessor));
        partialUpdateManager.add(Component.PROPERTY_BACKGROUND, new BackgroundUpdate(messageProcessor));
                
        partialUpdateManager.add(TextField.PROPERTY_INSETS,
                new InsetsUpdate(TextField.PROPERTY_INSETS, ".textarea", InsetsUpdate.CSS_PADDING));
                
        partialUpdateManager.add(TextField.TEXT_CHANGED_PROPERTY, new TextUpdate(messageProcessor));
        
    }

    /**
     * @see nextapp.echo2.webcontainer.ComponentSynchronizePeer#getContainerId(nextapp.echo2.app.Component)
     */
    public String getContainerId(Component child) {
        throw new UnsupportedOperationException("Component does not support children.");
    }

    /**
     * @see nextapp.echo2.webcontainer.image.ImageRenderSupport#getImage(nextapp.echo2.app.Component,
     *      java.lang.String)
     */

    public ImageReference getImage(Component component, String imageId) {
        if (IMAGE_ID_BACKGROUND.equals(imageId)) {
            FillImage backgroundImage;
            if (component.isRenderEnabled()) {
                backgroundImage = (FillImage) component.getRenderProperty(TextField.PROPERTY_BACKGROUND_IMAGE);
            } else {
                backgroundImage = (FillImage) component.getRenderProperty(TextField.PROPERTY_DISABLED_BACKGROUND_IMAGE);
                if (backgroundImage == null) {
                    backgroundImage = (FillImage) component.getRenderProperty(TextField.PROPERTY_BACKGROUND_IMAGE);
                }
            }
            if (backgroundImage == null) {
                return null;
            } else {
                return backgroundImage.getImage();
            }
        } else {
            return null;
        }
    }

    /**
     * @see nextapp.echo2.webcontainer.ActionProcessor#processAction(nextapp.echo2.webcontainer.ContainerInstance, 
     *      nextapp.echo2.app.Component, org.w3c.dom.Element)
     */
    public void processAction(ContainerInstance ci, Component component, Element actionElement) {
    
        String name = actionElement.getAttribute("name");
        String value = actionElement.getAttribute("value");
        
        ci.getUpdateManager().getClientUpdateManager().setComponentAction(component, name, value);

        //ci.getUpdateManager().getClientUpdateManager().setComponentAction(component, TextField.INPUT_ACTION, null);
    }

    /**
     * @see nextapp.echo2.webcontainer.PropertyUpdateProcessor#processPropertyUpdate(
     *      nextapp.echo2.webcontainer.ContainerInstance,
     *      nextapp.echo2.app.Component, org.w3c.dom.Element)
     */
    public void processPropertyUpdate(ContainerInstance ci, Component component, Element propertyElement) {
    
        String propertyName = propertyElement.getAttribute(PropertyUpdateProcessor.PROPERTY_NAME);
        
        if (TextField.TEXT_CHANGED_PROPERTY.equals(propertyName)) {
            String propertyValue = DomUtil.getElementText(propertyElement);
            ci.getUpdateManager().getClientUpdateManager().setComponentProperty(component, 
                    TextField.TEXT_CHANGED_PROPERTY, propertyValue);
        } else if (TextField.PROPERTY_HORIZONTAL_SCROLL.equals(propertyName)) {
            Extent propertyValue = new Extent(Integer.parseInt(
                    propertyElement.getAttribute(PropertyUpdateProcessor.PROPERTY_VALUE)));
            ci.getUpdateManager().getClientUpdateManager().setComponentProperty(component, 
                    TextField.PROPERTY_HORIZONTAL_SCROLL, propertyValue);
        } else if (TextField.PROPERTY_VERTICAL_SCROLL.equals(propertyName)) {
            Extent propertyValue = new Extent(Integer.parseInt(
                    propertyElement.getAttribute(PropertyUpdateProcessor.PROPERTY_VALUE)));
            ci.getUpdateManager().getClientUpdateManager().setComponentProperty(component, 
                    TextField.PROPERTY_VERTICAL_SCROLL, propertyValue);
        }
    }

    /**
     * @see nextapp.echo2.webcontainer.ComponentSynchronizePeer#renderAdd(nextapp.echo2.webcontainer.RenderContext,
     *      nextapp.echo2.app.update.ServerComponentUpdate, java.lang.String,
     *      nextapp.echo2.app.Component)
     */
    public void renderAdd(RenderContext rc, ServerComponentUpdate update, String targetId, Component component) {
        Element domAddElement = DomUpdate.renderElementAdd(rc.getServerMessage());
        DocumentFragment htmlFragment = rc.getServerMessage().getDocument().createDocumentFragment();
        renderHtml(rc, update, htmlFragment, component);
        DomUpdate.renderElementAddContent(rc.getServerMessage(), domAddElement, targetId, htmlFragment);
    }

    /**
     * @see nextapp.echo2.webcontainer.ComponentSynchronizePeer#renderDispose(nextapp.echo2.webcontainer.RenderContext,
     *      nextapp.echo2.app.update.ServerComponentUpdate,
     *      nextapp.echo2.app.Component)
     */
    public void renderDispose(RenderContext rc, ServerComponentUpdate update, Component component) {
        rc.getServerMessage().addLibrary(ACTIVE_TEXTAREA_SERVICE.getId());
        renderDisposeDirective(rc, (ActiveTextArea) component);
    }

    /**
     * Renders a directive to the outgoing <code>ServerMessage</code> to
     * dispose the state of a text component, performing tasks such as
     * registering event listeners on the client.
     * 
     * @param rc the relevant <code>RenderContext</code>
     * @param integerTextField the <code>LiveTextField<code>
     */
    public void renderDisposeDirective(RenderContext rc, ActiveTextArea integerTextField) {
        String elementId = ContainerInstance.getElementId(integerTextField);
        ServerMessage serverMessage = rc.getServerMessage();
        Element itemizedUpdateElement = serverMessage.getItemizedDirective(ServerMessage.GROUP_ID_PREREMOVE,
                "ActiveTextArea.MessageProcessor", "dispose", new String[0], new String[0]);
        Element itemElement = serverMessage.getDocument().createElement("item");
        itemElement.setAttribute("eid", elementId);
        itemizedUpdateElement.appendChild(itemElement);
    }

    /**
     * Renders a directive to the outgoing <code>ServerMessage</code> to
     * initialize the state of a text component, performing tasks such as
     * registering event listeners on the client.
     * 
     * @param rc the relevant <code>RenderContext</code>
     * @param integerTextField the <code>LiveTextField<code>
     */
    public void renderInitDirective(RenderContext rc, ActiveTextArea activeTextArea) {
    
        Extent horizontalScroll = (Extent) activeTextArea.getRenderProperty(ActiveTextArea.PROPERTY_HORIZONTAL_SCROLL);
        Extent verticalScroll = (Extent) activeTextArea.getRenderProperty(ActiveTextArea.PROPERTY_VERTICAL_SCROLL);
        
        String elementId = ContainerInstance.getElementId(activeTextArea);
        ServerMessage serverMessage = rc.getServerMessage();

        Element itemizedUpdateElement = serverMessage.getItemizedDirective(ServerMessage.GROUP_ID_POSTUPDATE,
                "ActiveTextArea.MessageProcessor", "init", new String[0], new String[0]);
                
        Element itemElement = serverMessage.getDocument().createElement("item");
        
        itemElement.setAttribute("eid", elementId);
        if (horizontalScroll != null && horizontalScroll.getValue() != 0) {
            itemElement.setAttribute("horizontal-scroll", ExtentRender.renderCssAttributePixelValue(horizontalScroll, "0"));
        }
        if (verticalScroll != null && verticalScroll.getValue() != 0) {
            itemElement.setAttribute("vertical-scroll", ExtentRender.renderCssAttributePixelValue(verticalScroll, "0"));
        }
        
        if (!activeTextArea.isRenderEnabled()) {
            itemElement.setAttribute("enabled", "false");
        }

        String text = activeTextArea.getText();
        if (text != null) 
            itemElement.setAttribute("text", text);

        itemElement.setAttribute("foreground", ColorRender.renderCssAttributeValue((Color)activeTextArea.getRenderProperty(Component.PROPERTY_FOREGROUND)));
        itemElement.setAttribute("background", ColorRender.renderCssAttributeValue((Color)activeTextArea.getRenderProperty(Component.PROPERTY_BACKGROUND)));


        itemElement.setAttribute("maximum-length", Integer.toString(activeTextArea.getMaxLength()));
        itemElement.setAttribute("msg-visible", activeTextArea.isMessageVisible() ? "true" : "false");
        
        
        if (activeTextArea.hasActionListeners())
            itemElement.setAttribute("server-notify", "true");

        itemizedUpdateElement.appendChild(itemElement);
    }

    /**
     * @see nextapp.echo2.webcontainer.FocusSupport#renderSetFocus(nextapp.echo2.webcontainer.RenderContext, 
     *      nextapp.echo2.app.Component)
     */
    public void renderSetFocus(RenderContext rc, Component component) {
        WindowUpdate.renderSetFocus(rc.getServerMessage(), ContainerInstance.getElementId(component));
    }

    /**
     * @see nextapp.echo2.webcontainer.ComponentSynchronizePeer#renderUpdate(nextapp.echo2.webcontainer.RenderContext,
     *      nextapp.echo2.app.update.ServerComponentUpdate, java.lang.String)
     */
    public boolean renderUpdate(RenderContext rc, ServerComponentUpdate update, String targetId) {
        boolean fullReplace = false;
        if (update.hasUpdatedProperties()) {
            if (!partialUpdateManager.canProcess(rc, update)) {
                fullReplace = true;
            }
        }

        if (fullReplace) {
            // Perform full update.
            DomUpdate.renderElementRemove(rc.getServerMessage(), ContainerInstance.getElementId(update.getParent()));
            renderAdd(rc, update, targetId, update.getParent());
        } else {
            partialUpdateManager.process(rc, update);
        }

        return false;
    }
    
    /**
     * @see nextapp.echo2.webcontainer.DomUpdateSupport#renderHtml(nextapp.echo2.webcontainer.RenderContext, 
     *      nextapp.echo2.app.update.ServerComponentUpdate, org.w3c.dom.Node, nextapp.echo2.app.Component)
     */
    public void renderHtml(RenderContext rc, ServerComponentUpdate addUpdate, Node parentNode, Component component) {
    
        ServerMessage serverMessage = rc.getServerMessage();
        serverMessage.addLibrary(ACTIVE_TEXTAREA_SERVICE.getId());
        
        ActiveTextArea activeTextArea = (ActiveTextArea) component;
        String elementId = ContainerInstance.getElementId(activeTextArea);


        // Create the outer <div> with element id
        
 		Element divOuter = parentNode.getOwnerDocument().createElement("div");
		divOuter.setAttribute("id", elementId);

        // Create the component parts
        Element textAreaElement = parentNode.getOwnerDocument().createElement("textarea");
		textAreaElement.setAttribute("id", elementId + ".textarea");

        Extent width = (Extent) activeTextArea.getRenderProperty(TextComponent.PROPERTY_WIDTH);
        Extent height = (Extent) activeTextArea.getRenderProperty(TextComponent.PROPERTY_HEIGHT);

        if (width != null) 
            textAreaElement.setAttribute("cols", ExtentRender.renderCssAttributeValue(width));

        if (height != null) 
            textAreaElement.setAttribute("rows", ExtentRender.renderCssAttributeValue(height));
  
  
        String text = activeTextArea.getText();
        if (text != null) {
            Text textNode = parentNode.getOwnerDocument().createTextNode(text);
            textAreaElement.appendChild(textNode);
        }
        
        if (activeTextArea.isFocusTraversalParticipant()) {
            textAreaElement.setAttribute("tabindex", Integer.toString(activeTextArea.getFocusTraversalIndex()));
        } else {
            textAreaElement.setAttribute("tabindex", "-1");
        }
        
        String toolTipText = (String) activeTextArea.getRenderProperty(TextArea.PROPERTY_TOOL_TIP_TEXT);
        if (toolTipText != null) {
            textAreaElement.setAttribute("title", toolTipText);
        }

        CssStyle cssStyle = createBaseCssStyle(rc, activeTextArea);
        
        if (cssStyle.hasAttributes())
            textAreaElement.setAttribute("style", cssStyle.renderInline());

        // 'length' span      
        Element lengthSpan = parentNode.getOwnerDocument().createElement("span");
		lengthSpan.setAttribute("id", elementId + ".length");
		cssStyle = new CssStyle();
		cssStyle.setAttribute("font-family", "sans-serif");
		cssStyle.setAttribute("font-size", "x-small");
		cssStyle.setAttribute("display", (activeTextArea.isMessageVisible() ? "" : "none"));
		lengthSpan.setAttribute("style", cssStyle.renderInline());

        // 'remaining' span      
        Element remainingSpan = parentNode.getOwnerDocument().createElement("span");
		remainingSpan.setAttribute("id", elementId + ".remaining");
		cssStyle = new CssStyle();
		cssStyle.setAttribute("padding-left", "1em");
		cssStyle.setAttribute("font-family", "sans-serif");
		cssStyle.setAttribute("font-size", "x-small");
		cssStyle.setAttribute("display", (activeTextArea.isMessageVisible() ? "" : "none"));
		remainingSpan.setAttribute("style", cssStyle.renderInline());

        // 'overdrawn' span      
        Element overdrawnSpan = parentNode.getOwnerDocument().createElement("span");
		overdrawnSpan.setAttribute("id", elementId + ".overdrawn");
		cssStyle = new CssStyle();
		cssStyle.setAttribute("padding-left", "1em");
		cssStyle.setAttribute("font-family", "sans-serif");
		cssStyle.setAttribute("font-size", "x-small");
		cssStyle.setAttribute("color", "#ff0000");
		cssStyle.setAttribute("font-weight", "bold");
		cssStyle.setAttribute("display", (activeTextArea.isMessageVisible() ? "" : "none"));
		overdrawnSpan.setAttribute("style", cssStyle.renderInline());


        // Assemble the component parts starting with the <input>

 		Element divTextArea = parentNode.getOwnerDocument().createElement("div");

		parentNode.appendChild(divOuter);
		divOuter.appendChild(divTextArea);
		divTextArea.appendChild(textAreaElement);

 		Element divMessage = parentNode.getOwnerDocument().createElement("div");
		cssStyle = new CssStyle();
		cssStyle.setAttribute("margin-top", "3px");
		divMessage.setAttribute("style", cssStyle.renderInline());
 		
		divMessage.appendChild(lengthSpan);
		divMessage.appendChild(remainingSpan);
		divMessage.appendChild(overdrawnSpan);
		divOuter.appendChild(divMessage);

        renderInitDirective(rc, (ActiveTextArea)activeTextArea);
    }


    /**
     * Creates a base <code>CssStyle</code> for properties common to text
     * components.
     * 
     * @param rc the relevant <code>RenderContext</code>
     * @param integerTextField the text component
     * @return the style
     */

    protected CssStyle createBaseCssStyle(RenderContext rc, ActiveTextArea activeTextArea) {
        CssStyle cssStyle = new CssStyle();

        boolean renderEnabled = activeTextArea.isRenderEnabled();

        Border border;
        Color foreground, background;
        Font font;
        FillImage backgroundImage;
        if (!renderEnabled) {
            // Retrieve disabled style information.
            background = (Color) activeTextArea.getRenderProperty(TextField.PROPERTY_DISABLED_BACKGROUND);
            backgroundImage = (FillImage) activeTextArea.getRenderProperty(TextField.PROPERTY_DISABLED_BACKGROUND_IMAGE);
            border = (Border) activeTextArea.getRenderProperty(TextField.PROPERTY_DISABLED_BORDER);
            font = (Font) activeTextArea.getRenderProperty(TextField.PROPERTY_DISABLED_FONT);
            foreground = (Color) activeTextArea.getRenderProperty(TextField.PROPERTY_DISABLED_FOREGROUND);

            // Fallback to normal styles.
            if (background == null) {
                background = (Color) activeTextArea.getRenderProperty(Component.PROPERTY_BACKGROUND);
                if (backgroundImage == null) {
                    // Special case:
                    // Disabled background without disabled background image will render disabled background instead of
                    // normal background image.
                    backgroundImage = (FillImage) activeTextArea.getRenderProperty(TextField.PROPERTY_BACKGROUND_IMAGE);
                }
            }
            if (border == null) {
                border = (Border) activeTextArea.getRenderProperty(TextField.PROPERTY_BORDER);
            }
            if (font == null) {
                font = (Font) activeTextArea.getRenderProperty(TextField.PROPERTY_FONT);
            }
            if (foreground == null) {
                foreground = (Color) activeTextArea.getRenderProperty(Component.PROPERTY_FOREGROUND);
            }
        } else {
            border = (Border) activeTextArea.getRenderProperty(TextField.PROPERTY_BORDER);
            foreground = (Color) activeTextArea.getRenderProperty(Component.PROPERTY_FOREGROUND);
            background = (Color) activeTextArea.getRenderProperty(Component.PROPERTY_BACKGROUND);
            font = (Font) activeTextArea.getRenderProperty(TextField.PROPERTY_FONT);
        }
        
        Alignment alignment = (Alignment) activeTextArea.getRenderProperty(TextField.PROPERTY_ALIGNMENT);
        
        if (alignment != null) {
            int horizontalAlignment = AlignmentRender.getRenderedHorizontal(alignment, activeTextArea);
            switch (horizontalAlignment) {
            case Alignment.LEFT:
                cssStyle.setAttribute("text-align", "left");
                break;
            case Alignment.CENTER:
                cssStyle.setAttribute("text-align", "center");
                break;
            case Alignment.RIGHT:
                cssStyle.setAttribute("text-align", "right");
                break;
            }
        }
        
        BorderRender.renderToStyle(cssStyle, border);
        ColorRender.renderToStyle(cssStyle, foreground, background);
        FontRender.renderToStyle(cssStyle, font);
        
        
        InsetsRender.renderToStyle(cssStyle, "padding", (Insets) activeTextArea.getRenderProperty(TextField.PROPERTY_INSETS));
        
        Extent width = (Extent) activeTextArea.getRenderProperty(TextArea.PROPERTY_WIDTH);
        Extent height = (Extent) activeTextArea.getRenderProperty(TextArea.PROPERTY_HEIGHT);

        if (width != null) {
            cssStyle.setAttribute("width", ExtentRender.renderCssAttributeValue(width));
        }

        if (height != null) {
            cssStyle.setAttribute("height", ExtentRender.renderCssAttributeValue(height));
        }
        return cssStyle;
    }


    static int convertAlignmentToPosition(Alignment position, Component component) {

        if (position.getVertical() == Alignment.DEFAULT) {
        
            switch (position.getHorizontal()) {
                case Alignment.LEFT:
                    return Alignment.LEFT;
                case Alignment.RIGHT:
                    return Alignment.RIGHT;
                case Alignment.LEADING:
                    return component.getRenderLayoutDirection().isLeftToRight() 
                        ? Alignment.LEFT : Alignment.RIGHT;
                case Alignment.TRAILING:
                    return component.getRenderLayoutDirection().isLeftToRight() 
                        ? Alignment.RIGHT : Alignment.LEFT;
                default:
                    // Invalid, return value for TRAILING (default).
                    return Alignment.RIGHT;
            }
        } else {
            if (position.getVertical() == Alignment.TOP) {
                return Alignment.TOP;
            } else {
                return Alignment.BOTTOM;
            }
        }
    }


}
