package com.informagen.echo2.peers;

// This peer component
import com.informagen.echo2.HorizontalRule;

import nextapp.echo2.app.Border;
import nextapp.echo2.app.Color;
import nextapp.echo2.app.Component;
import nextapp.echo2.app.Extent;
import nextapp.echo2.app.Insets;

import nextapp.echo2.app.update.ServerComponentUpdate;

import nextapp.echo2.webcontainer.ComponentSynchronizePeer;
import nextapp.echo2.webcontainer.ContainerInstance;
import nextapp.echo2.webcontainer.DomUpdateSupport;
import nextapp.echo2.webcontainer.RenderContext;

// CSS property renderers
import nextapp.echo2.webcontainer.propertyrender.BorderRender;
import nextapp.echo2.webcontainer.propertyrender.ColorRender;
import nextapp.echo2.webcontainer.propertyrender.ExtentRender;
import nextapp.echo2.webcontainer.propertyrender.InsetsRender;

import nextapp.echo2.webrender.output.CssStyle;
import nextapp.echo2.webrender.servermessage.DomUpdate;

// W3C
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * <code>HorizontalRulePeer</code>
 */

public class HorizontalRulePeer implements ComponentSynchronizePeer, DomUpdateSupport {


    /**
    * @see nextapp.echo2.webcontainer.ComponentSynchronizePeer#getContainerId(nextapp.echo2.app.Component)
    */
    
    public String getContainerId(Component child) {
        throw new IllegalStateException("HorizontalRulePeer does not work as a DOM container");
    }
    
    
    /**
    * @see nextapp.echo2.webcontainer.ComponentSynchronizePeer#renderAdd(nextapp.echo2.webcontainer.RenderContext,
    *      nextapp.echo2.app.update.ServerComponentUpdate, java.lang.String,
    *      nextapp.echo2.app.Component)
    */
    
    public void renderAdd(RenderContext rc, ServerComponentUpdate update, String targetId, Component component) {
    
        Element domAddElement = DomUpdate.renderElementAdd(rc.getServerMessage());
        DocumentFragment htmlFragment = rc.getServerMessage().getDocument().createDocumentFragment();
        
        renderHtml(rc, update, htmlFragment, component);
        
        DomUpdate.renderElementAddContent(rc.getServerMessage(), domAddElement, targetId, htmlFragment);
    }
    
    
    /**
    * @see nextapp.echo2.webcontainer.ComponentSynchronizePeer#renderDispose(nextapp.echo2.webcontainer.RenderContext,
    *      nextapp.echo2.app.update.ServerComponentUpdate,
    *      nextapp.echo2.app.Component)
    */
    
    public void renderDispose(RenderContext rc, ServerComponentUpdate update, Component component) {
        // we dont have anything to dispose of per see
    }
    
    
    /**
    * @see nextapp.echo2.webcontainer.ComponentSynchronizePeer#renderUpdate(nextapp.echo2.webcontainer.RenderContext,
    *      nextapp.echo2.app.update.ServerComponentUpdate, java.lang.String)
    */
    
    public boolean renderUpdate(RenderContext rc, ServerComponentUpdate update, String targetId) {
    
        DomUpdate.renderElementRemove(rc.getServerMessage(), ContainerInstance.getElementId(update.getParent()));
        renderAdd(rc, update, targetId, update.getParent());
        
        return false;
    }


    /**
    * @see nextapp.echo2.webcontainer.DomUpdateSupport#renderHtml(nextapp.echo2.webcontainer.RenderContext,
    *      nextapp.echo2.app.update.ServerComponentUpdate, org.w3c.dom.Node,
    *      nextapp.echo2.app.Component)
    */
    
    public void renderHtml(RenderContext rc, ServerComponentUpdate update, Node parentNode, Component component) {
    
        Document doc = rc.getServerMessage().getDocument();
        Element element = doc.createElement("hr");
        element.setAttribute("id",ContainerInstance.getElementId(component));
        
        CssStyle cssStyle = createStyle(component);
        
        if (cssStyle.hasAttributes()) 
            element.setAttribute("style", cssStyle.renderInline());
        
        parentNode.appendChild(element);
    }
        
        
    private CssStyle createStyle(Component component) {
    
        CssStyle cssStyle = new CssStyle();
        
        // 'width' & 'height' ==============================================================
        
        Extent width = (Extent) component.getRenderProperty(HorizontalRule.PROPERTY_WIDTH);
        Extent height = (Extent) component.getRenderProperty(HorizontalRule.PROPERTY_HEIGHT);

        if (width != null) 
            cssStyle.setAttribute("width", ExtentRender.renderCssAttributeValue(width));
        
        if (height != null) 
            cssStyle.setAttribute("height", ExtentRender.renderCssAttributeValue(height));


        // 'background-color' ==============================================================
        
        Color color = (Color) component.getRenderProperty(Component.PROPERTY_BACKGROUND);

        if(color == null)
            color = (Color) component.getRenderProperty(Component.PROPERTY_FOREGROUND);

        if( component.isRenderEnabled() == false)
            color = (Color) component.getRenderProperty(HorizontalRule.PROPERTY_DISABLED_BACKGROUND);

        if(color != null)        
            ColorRender.renderToStyle(cssStyle, color, color);

        // 'border' ========================================================================
        
        Border border = (Border)component.getRenderProperty(HorizontalRule.PROPERTY_BORDER);

        if( component.isRenderEnabled() == false)
            border = (Border) component.getRenderProperty(HorizontalRule.PROPERTY_DISABLED_BORDER);

        if(border != null)
            BorderRender.renderToStyle(cssStyle, border);
        
        return cssStyle;
    }
    
        
}