package com.informagen.echo2.peers;


// Our Echo2 component
import com.informagen.echo2.RegExTextField;

// NextApp Echo2 Component Framework
import nextapp.echo2.app.Component;

import nextapp.echo2.app.update.ServerComponentUpdate;


// NextApp Echo2 WebContainer Framework
import nextapp.echo2.webcontainer.RenderContext;
import nextapp.echo2.webcontainer.DomUpdateSupport;


// NextApp Echo2 WebRenderer Sub-Framework
import nextapp.echo2.webrender.ServerMessage;
import nextapp.echo2.webrender.Service;
import nextapp.echo2.webrender.ServiceRegistry;
import nextapp.echo2.webrender.WebRenderServlet;

import nextapp.echo2.webrender.service.JavaScriptService;


// W3C DOM
import org.w3c.dom.Node;
import org.w3c.dom.Element;


/**
 * Synchronization peer for the 
 * <code>com.informagen.echo2.IntegerTextField</code> component
 */


public class RegExTextFieldPeer extends ActiveTextFieldPeer
                                implements DomUpdateSupport {

    /**
     * Service to provide supporting JavaScript library.
     */

    static final Service REGEX_TEXTFIELD_SERVICE = JavaScriptService.forResource(
            "Echo.RegExTextField",
            "/com/informagen/echo2/resources/js/RegExTextField.js");


    static {
		ServiceRegistry registery = WebRenderServlet.getServiceRegistry();
        registery.add(REGEX_TEXTFIELD_SERVICE);
    }
 

    /**
     * Default constructor.
     */

    public RegExTextFieldPeer() {
        super("RegExTextField.MessageProcessor");
    }

    /**
     * @see nextapp.echo2.webcontainer.ComponentSynchronizePeer#renderDispose(nextapp.echo2.webcontainer.RenderContext,
     *      nextapp.echo2.app.update.ServerComponentUpdate,
     *      nextapp.echo2.app.Component)
     */

    public void renderDispose(RenderContext rc, ServerComponentUpdate update, Component component) {

        ServerMessage serverMessage = rc.getServerMessage();
        serverMessage.addLibrary(REGEX_TEXTFIELD_SERVICE.getId());

        super.renderDispose(rc, update, component);
    }


//         if (regexTextField.getRegExFilter() != null && !regexTextField.getRegExFilter().equals("")) 
//             itemElement.setAttribute("regexp-filter", regexTextField.getRegExFilter());
//         
//         if (regexTextField.getRegEx() != null && !regexTextField.getRegEx().equals("")) 
//             itemElement.setAttribute("regexp", regexTextField.getRegEx());




    
    /**
     * @see nextapp.echo2.webcontainer.DomUpdateSupport#renderHtml(nextapp.echo2.webcontainer.RenderContext, 
     *      nextapp.echo2.app.update.ServerComponentUpdate, org.w3c.dom.Node, nextapp.echo2.app.Component)
     */
    public void renderHtml(RenderContext rc, ServerComponentUpdate addUpdate, Node parentNode, Component component) {
    
        ServerMessage serverMessage = rc.getServerMessage();
        serverMessage.addLibrary(REGEX_TEXTFIELD_SERVICE.getId());
        
        RegExTextField regexTextField = (RegExTextField)component;
        
        super.renderHtml(rc, addUpdate, parentNode, regexTextField);


        Element itemElement = renderInitDirective(rc, regexTextField);
        
        if (regexTextField.getRegExFilter() != null && !regexTextField.getRegExFilter().equals("")) 
            itemElement.setAttribute("regexp-filter", regexTextField.getRegExFilter());
        
        if (regexTextField.getRegEx() != null && !regexTextField.getRegEx().equals("")) 
            itemElement.setAttribute("regexp", regexTextField.getRegEx());

    }



}
