package com.informagen.starter.echo;

import nextapp.echo2.testapp.interactive.Styles;
import com.informagen.starter.echo.ServletListener;


import nextapp.echo2.app.ApplicationInstance;

// Servlet interaction
import nextapp.echo2.webcontainer.command.BrowserRedirectCommand;
import nextapp.echo2.webcontainer.ContainerContext;

import javax.servlet.http.HttpSession;

import org.apache.log4j.Logger;


// Java Util
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;


/**
 * Inquiry/Encounter Application Instance.
 */
 
abstract public class EchoApplication extends ApplicationInstance {

	private static Logger logger = Logger.getLogger(EchoApplication.class.getName());

    private ContainerContext ctx = null;
    private Properties requestProperties = null;

    protected void initApplication() {
    
        // Register this application with any servlet listeners
        registerWithServletListener();
        
        // Load the StyleSheets
        setStyleSheet(Styles.DEFAULT_STYLE_SHEET);
    }

    /**
     * Convenience method to return the active Inquiry/Design application as a
     * <code>InquiryDesignApplication</code>.
     * 
     * @return the active <code>InquiryDesignApplication</code>
     */

    public static EchoApplication getApp() {
        return (EchoApplication) getActive();
    }


    public Properties requestProperties() {
        if (requestProperties == null) {
            this.requestProperties = collectRequestParameters();
        }
        
        return this.requestProperties;
    }

    private ContainerContext getContainerContext() {
        if (this.ctx == null) {
            String ctxName = ContainerContext.CONTEXT_PROPERTY_NAME;
            this.ctx = (ContainerContext) getContextProperty(ctxName);
        }
        
        return this.ctx;
    }
    


    /*
     *  Register this application instance with the Servlet listener in order
     *    to shut down this instance gracefully, closing all open resources
     */

    protected void registerWithServletListener() {

		HttpSession httpSession = getContainerContext().getSession();
		
		// Jetty return 'null'
		if (httpSession != null) 
            httpSession.setAttribute(ServletListener.APPLICATION_INSTANCE, this);
    }

    /*
     *  Create a set of the servlet request parameters
     */

    private Properties collectRequestParameters() {
 
		Map requestParameters = getContainerContext().getInitialRequestParameterMap();
		Set keys = requestParameters.keySet();
        Properties properties = new Properties();
		
		Iterator itor = keys.iterator();
		
		while(itor.hasNext()) {
		    String key = (String)itor.next();
		    String[] values = (String[] )requestParameters.get(key);
		    properties.setProperty(key, values[0]);
		}
                
        return properties;
    }


    // Error handling ============================================================

    public static void reportError(Exception errorException) {
        reportError(errorException.getMessage());
    }

    public static void reportError(String errorMessage) {
        getApp().showErrorMessage(errorMessage);
    }

    public void showErrorMessage(String errorMessage) {
        // Optionally put up a Message window dialog
        logger.error(errorMessage);
    }


    // Shutdown ==================================================================

    public void shutdown() {
        logger.debug("Invoke the shutdown servlet");
        releaseResources();
        
        
        // Remove the reference to the Application so that we won't
        //  invoke 'releaseResource' a second time
        
 		HttpSession session = getContainerContext().getSession();
 		
		// Jetty return 'null'
        if(session != null)
 		    session.removeAttribute(ServletListener.APPLICATION_INSTANCE);
        
        // The 'shutdown' servlet is the only way to properly
        //  close down an Echo servlet
        enqueueCommand(new BrowserRedirectCommand("shutdown"));
    }
 

    public void releaseResources() {
        logger.debug("No resources to be released");
    }


}
