/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2005 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo2.testapp.interactive;

import nextapp.echo2.app.Button;
import nextapp.echo2.app.Component;
import nextapp.echo2.app.ContentPane;
import nextapp.echo2.app.Extent;
import nextapp.echo2.app.Label;
import nextapp.echo2.app.Column;
import nextapp.echo2.app.SplitPane;
import nextapp.echo2.app.event.ActionEvent;
import nextapp.echo2.app.event.ActionListener;

import com.informagen.starter.echo.EchoApplication;

import org.apache.log4j.Logger;


/**
 * Main InteractiveTest <code>ContentPane</code> which displays a menu
 * of available tests.
 */
public class TestPane extends ContentPane {

	private static Logger logger = Logger.getLogger(TestPane.class.getName());


    private SplitPane horizontalPane;
    private Column testButtonsColumn;
    
    public TestPane() {
        super();
        
        SplitPane verticalPane = new SplitPane(SplitPane.ORIENTATION_VERTICAL);
        verticalPane.setStyleName("TestPane");
        add(verticalPane);

        Label titleLabel = new Label("Echo2 Component Test Harness");
        titleLabel.setStyleName("TitleLabel");
        verticalPane.add(titleLabel);
        
        horizontalPane = new SplitPane(SplitPane.ORIENTATION_HORIZONTAL, new Extent(215));
        horizontalPane.setStyleName("DefaultResizable");
        verticalPane.add(horizontalPane);
        
        Column controlsColumn = new Column();
        controlsColumn.setStyleName("ApplicationControlsColumn");
        controlsColumn.setCellSpacing(new Extent(12));
        
        horizontalPane.add(controlsColumn);
        
        testButtonsColumn = new Column();
        testButtonsColumn.setCellSpacing(new Extent(2));
        controlsColumn.add(testButtonsColumn);

        // Component tests, button label and test action
        addTest("HorizontalRule",   "HorizontalRuleTest");
        addTest("RegExTextField",   "RegExTextFieldTest");
        addTest("IntegerTextField", "IntegerTextFieldTest");
        addTest("NumericTextField", "NumericTextFieldTest");
        addTest("ActiveTextArea",   "ActiveTextAreaTest");


        // Console Button ==================================================
        Button consoleBtn = new Button("Console");
        consoleBtn.setStyleName("Default");
        controlsColumn.add(consoleBtn);

        consoleBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
            ((InteractiveApp) getApplicationInstance()).setConsoleVisible(true);
            }
        });


        // Exit Button =====================================================
        Button exitBtn = new Button("Exit");
        exitBtn.setStyleName("Default");
        controlsColumn.add(exitBtn);

        exitBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                doQuit();
            }
        });
    }
    
    private void addTest(String name, String action) {
        Button button = new Button(name);
        button.setRenderId("StartTest" + action);
        button.setId("StartTest:" + action);
        button.setActionCommand(action);
        button.setStyleName("Default");
        button.addActionListener(commandActionListener);
        testButtonsColumn.add(button);
    }

    // Action listener fullfilment methods /////////////////////////////////////////

    protected void doQuit() {
        InteractiveApp.getApp().shutdown();
    }



    // Inner class action listener ////////////////////////////////////////////////

    private ActionListener commandActionListener = new ActionListener() {
        
        private Button activeButton = null;
        
        /**
         * @see nextapp.echo2.app.event.ActionListener#actionPerformed(nextapp.echo2.app.event.ActionEvent)
         */
        public void actionPerformed(ActionEvent e) {
        
        
            try {
            
                if (activeButton != null) 
                    activeButton.setStyleName("Default");
                
                Button button = (Button) e.getSource();
                button.setStyleName("Selected");
                activeButton = button;

                String screenClassName = "nextapp.echo2.testapp.interactive.testscreen." + e.getActionCommand();
                Class screenClass = Class.forName(screenClassName);
                Component content = (Component) screenClass.newInstance();
                
                // Remove any existing component from the right pane.  The left side
                //  has the index of 0
                
                if (horizontalPane.getComponentCount() > 1)
                    horizontalPane.remove(1);
    
                // Add new test content
                horizontalPane.add(content);
                
            } catch (ClassNotFoundException ex) {
                logger.error(ex.toString());
                activeButton.setStyleName("Default");
            } catch (InstantiationException ex) {
                logger.error(ex.toString());
                activeButton.setStyleName("Default");
            } catch (IllegalAccessException ex) {
                logger.error(ex.toString());
                activeButton.setStyleName("Default");
            }
        }
    };


}
