/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2005 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo2.testapp.interactive.testscreen;

import nextapp.echo2.app.Alignment;
import nextapp.echo2.app.Border;
import nextapp.echo2.app.Column;
import nextapp.echo2.app.Component;
import nextapp.echo2.app.Color;
import nextapp.echo2.app.Extent;
import nextapp.echo2.app.Label;
import nextapp.echo2.app.Insets;
import nextapp.echo2.app.SplitPane;
import nextapp.echo2.app.text.Document;

import nextapp.echo2.app.event.ActionEvent;
import nextapp.echo2.app.event.ActionListener;
import nextapp.echo2.app.event.DocumentEvent;
import nextapp.echo2.app.event.DocumentListener;

import nextapp.echo2.testapp.interactive.ButtonColumn;
import nextapp.echo2.testapp.interactive.InteractiveApp;
import nextapp.echo2.testapp.interactive.StyleUtil;
import nextapp.echo2.testapp.interactive.Styles;

import com.informagen.echo2.ActiveTextField;


abstract public class ActiveTextFieldTest extends SplitPane {
    
    /**
     * Writes <code>ActionEvent</code>s to console.
     */
    private ActionListener actionListener = new ActionListener() {

        /**
         * @see nextapp.echo2.app.event.ActionListener#actionPerformed(nextapp.echo2.app.event.ActionEvent)
         */
        public void actionPerformed(ActionEvent e) {
            String action = e.getActionCommand();
            ((InteractiveApp) getApplicationInstance()).consoleWrite(action);
        }
    };
    
    
    /**
     * Writes <code>ActionEvent</code>s to console.
     */

    private DocumentListener documentListener = new DocumentListener() {
        
        /**
         * @see nextapp.echo2.app.event.DocumentListener#documentUpdate(nextapp.echo2.app.event.DocumentEvent)
         */
        public void documentUpdate(DocumentEvent e) {
            ((InteractiveApp) getApplicationInstance()).consoleWrite(e.toString());
            ((InteractiveApp) getApplicationInstance()).consoleWrite(((Document) e.getSource()).getText());
        }
    };




    protected ActiveTextField activeTextField;
    protected Column controlGroupsColumn;
    
    public ActiveTextFieldTest(ActiveTextField activeTextField) {
        super(SplitPane.ORIENTATION_HORIZONTAL, new Extent(250, Extent.PX));
        
        this.activeTextField = activeTextField;
 
        setStyleName("DefaultResizable");
        
        
        // Build Test Control Column, subclass will fill it
        controlGroupsColumn = new Column();
        controlGroupsColumn.setStyleName("TestControlsColumn");
        add(controlGroupsColumn);
        
        // Build the testing panel in the splitpane's right side
        add(buildTestingPanel());
        
    }


    protected void fillTestControlsColumn() {        
        
        // Control Groups in the splitpane's left side


        // Build the test control buttons as small groups (controlsColumn) 
        //   placed within a larger group (controlGroupsColumn) 
        ButtonColumn controlsColumn;

                
        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Set Text"));
        

        controlsColumn.addButton("Text -> 10", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean enabled = !activeTextField.isMessageVisible();
                activeTextField.setText("10");
            }
        });

        controlsColumn.addButton("Text -> 10.0", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean enabled = !activeTextField.isMessageVisible();
                activeTextField.setText("10.0");
            }
        });

        controlsColumn.addButton("Empty String", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setText("");
            }
        });
        
        controlsColumn.addButton("null", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setText(null);
            }
        });

        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Icon/Message"));
        

        controlsColumn.addButton("Toggle Message Visible", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean enabled = !activeTextField.isMessageVisible();
                activeTextField.setMessageVisible(enabled);
            }
        });

        controlsColumn.addButton("Toggle Icon Visible", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean enabled = !activeTextField.isIconVisible();
                activeTextField.setIconVisible(enabled);
            }
        });

        controlsColumn.addButton("Toggle Message Always Visible", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean enabled = !activeTextField.isMessageAlwaysVisible();
                activeTextField.setMessageAlwaysVisible(enabled);
            }
        });

        controlsColumn.addButton("Toggle Icon Always Visible", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean enabled = !activeTextField.isIconAlwaysVisible();
                activeTextField.setIconAlwaysVisible(enabled);
            }
        });



        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Width"));

        controlsColumn.addButton("Width -> null", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setWidth(null);
            }
        });

        controlsColumn.addButton("Width -> 200px", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setWidth(new Extent(200, Extent.PX));
            }
        });

        controlsColumn.addButton("Width -> 100%", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setWidth(new Extent(100, Extent.PERCENT));
            }
        });



        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Listeners"));

        controlsColumn.addButton("Get Text", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String text = activeTextField.getText();
                ((InteractiveApp) getApplicationInstance()).consoleWrite(text);
            }
        });
        
        controlsColumn.addButton("Add ActionListener", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.addActionListener(actionListener);
            }
        });
        
        controlsColumn.addButton("Remove ActionListener", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.removeActionListener(actionListener);
            }
        });

        controlsColumn.addButton("Add DocumentListener", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.getDocument().addDocumentListener(documentListener);
            }
        });
        
        controlsColumn.addButton("Remove DocumentListener", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.getDocument().removeDocumentListener(documentListener);
            }
        });


/*
        
        controlsColumn.addButton("Change Border (All Attributes)", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Border border = StyleUtil.randomBorder();
                activeTextField.setBorder(border);
            }
        });

        controlsColumn.addButton("Change Border Color", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Border border = activeTextField.getBorder();
                if (border == null) {
                    return;
                }
                border = new Border(border.getSize(), StyleUtil.randomColor(), border.getStyle());
                activeTextField.setBorder(border);
            }
        });

        controlsColumn.addButton("Change Border Size", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Border border = StyleUtil.nextBorderSize(activeTextField.getBorder());
                if (border == null) {
                    return;
                }
                activeTextField.setBorder(border);
            }
        });

        controlsColumn.addButton("Change Border Style", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Border border = StyleUtil.nextBorderStyle(activeTextField.getBorder());
                if (border == null) {
                    return;
                }
                activeTextField.setBorder(border);
            }
        });




        controlsColumn.addButton("Toggle Background Image", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                FillImage backgroundImage = activeTextField.getBackgroundImage();
                if (backgroundImage == null) {
                    activeTextField.setBackgroundImage(Styles.BG_SHADOW_LIGHT_BLUE);
                } else {
                    activeTextField.setBackgroundImage(null);
                }
            }
        });



*/
        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Foreground & Background"));

        controlsColumn.addButton("Random Foreground", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Color color = StyleUtil.randomColor();
                activeTextField.setForeground(color);
            }
        });
        controlsColumn.addButton("Default Foreground", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setForeground(null);
            }
        });

        controlsColumn.addButton("Random Background", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Color color = StyleUtil.randomColor();
                activeTextField.setBackground(color);
            }
        });
        
        controlsColumn.addButton("Default Background", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setBackground(null);
            }
        });


        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Error Colors"));

        controlsColumn.addButton("Random Foreground", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Color color = StyleUtil.randomColor();
                activeTextField.setErrorForeground(color);
            }
        });

        controlsColumn.addButton("Default Foreground", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setErrorForeground(null);
            }
        });


        controlsColumn.addButton("Random Background", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Color color = StyleUtil.randomColor();
                activeTextField.setErrorBackground(color);
            }
        });

        controlsColumn.addButton("Default Background", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setErrorBackground(null);
            }
        });




/*
        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Enable/Disable"));

        controlsColumn.addButton("Toggle Enabled", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean enabled = !activeTextField.isEnabled();
                activeTextField.setEnabled(enabled);
            }
        });
        
        controlsColumn.addButton("Change Disabled Border (All Attributes)", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Border border = StyleUtil.randomBorder();
                activeTextField.setDisabledBorder(border);
            }
        });

        controlsColumn.addButton("Change Disabled Border Color", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Border border = activeTextField.getDisabledBorder();
                if (border == null) {
                    return;
                }
                border = new Border(border.getSize(), StyleUtil.randomColor(), border.getStyle());
                activeTextField.setDisabledBorder(border);
            }
        });
        controlsColumn.addButton("Change Disabled Border Size", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Border border = StyleUtil.nextBorderSize(activeTextField.getDisabledBorder());
                if (border == null) {
                    return;
                }
                activeTextField.setDisabledBorder(border);
            }
        });
        controlsColumn.addButton("Change Disabled Border Style", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Border border = StyleUtil.nextBorderStyle(activeTextField.getDisabledBorder());
                if (border == null) {
                    return;
                }
                activeTextField.setDisabledBorder(border);
            }
        });

        controlsColumn.addButton("Toggle Disabled Background Image", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                FillImage backgroundImage = activeTextField.getDisabledBackgroundImage();
                if (backgroundImage == null) {
                    activeTextField.setDisabledBackgroundImage(Styles.BG_SHADOW_LIGHT_BLUE);
                } else {
                    activeTextField.setDisabledBackgroundImage(null);
                }
            }
        });
        controlsColumn.addButton("Set Disabled Foreground", new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                Color color = StyleUtil.randomColor();
                activeTextField.setDisabledForeground(color);
            }
        });

        controlsColumn.addButton("Clear Disabled Foreground", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setDisabledForeground(null);
            }
        });
        controlsColumn.addButton("Set Disabled Background", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Color color = StyleUtil.randomColor();
                activeTextField.setDisabledBackground(color);
            }
        });

        controlsColumn.addButton("Clear Disabled Background", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setDisabledBackground(null);
            }
        });


        controlsColumn.addButton("Insets -> null", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setInsets(null);
            }
        });

        controlsColumn.addButton("Insets -> 0px", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setInsets(new Insets(0));
            }
        });

        controlsColumn.addButton("Insets -> 5px", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setInsets(new Insets(5));
            }
        });

        controlsColumn.addButton("Insets -> 10/20/30/40px", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setInsets(new Insets(10, 20, 30, 40));
            }
        });
*/

        // Icon Position

        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Icon Position"));

        controlsColumn.addButton("Default", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setIconPosition(null);
            }
        });
        controlsColumn.addButton("Top", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setIconPosition(Alignment.ALIGN_TOP);
            }
        });
        controlsColumn.addButton("Bottom", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setIconPosition(Alignment.ALIGN_BOTTOM);
            }
        });
        controlsColumn.addButton("Left", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setIconPosition(Alignment.ALIGN_LEFT);
            }
        });
        controlsColumn.addButton("Right", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setIconPosition(Alignment.ALIGN_RIGHT);
            }
        });
        controlsColumn.addButton("Leading", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setIconPosition(new Alignment(Alignment.LEADING, Alignment.DEFAULT));
            }
        });
        controlsColumn.addButton("Trailing", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setIconPosition(new Alignment(Alignment.TRAILING, Alignment.DEFAULT));
            }
        });

        // Message Position

        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Status Message Position"));

        controlsColumn.addButton("Default", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setMessagePosition(null);
            }
        });
        controlsColumn.addButton("Top", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setMessagePosition(Alignment.ALIGN_TOP);
            }
        });
        controlsColumn.addButton("Bottom", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setMessagePosition(Alignment.ALIGN_BOTTOM);
            }
        });
        controlsColumn.addButton("Left", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setMessagePosition(Alignment.ALIGN_LEFT);
            }
        });
        controlsColumn.addButton("Right", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setMessagePosition(Alignment.ALIGN_RIGHT);
            }
        });
        controlsColumn.addButton("Leading", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setMessagePosition(new Alignment(Alignment.LEADING, Alignment.DEFAULT));
            }
        });
        controlsColumn.addButton("Trailing", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextField.setMessagePosition(new Alignment(Alignment.TRAILING, Alignment.DEFAULT));
            }
        });



        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("ToolTip"));
        
        controlsColumn.addButton("Toggle ToolTip Enabled", new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                if (activeTextField.getToolTipText() == null) {
                    activeTextField.setToolTipText("Integer TextField ");
                } else {
                    activeTextField.setToolTipText(null);
                }
            }
        });

    }


    private Component buildTestingPanel() {
    
        Column testColumn = new Column();
        testColumn.setStyleName("TestColumn");
        testColumn.add(activeTextField);
        
        return testColumn;
    }


}
